@@ -9,8 +9,6 @@
 #include "opencv2/core/eigen.hpp"
 #endif
 
-#include "opencv2/core/cuda.hpp"
-
 namespace opencv_test { namespace {
 
 class Core_ReduceTest : public cvtest::BaseTest
@@ -28,7 +26,7 @@ protected:
 template<class Type>
 void testReduce( const Mat& src, Mat& sum, Mat& avg, Mat& max, Mat& min, int dim )
 {
-    CV_Assert( src.channels() == 1 );
+    assert( src.channels() == 1 );
     if( dim == 0 ) // row
     {
         sum.create( 1, src.cols, CV_64FC1 );
@@ -138,7 +136,7 @@ int Core_ReduceTest::checkOp( const Mat& src, int dstType, int opType, const Mat
             eps = 0.6;
     }
 
-    CV_Assert( opRes.type() == CV_64FC1 );
+    assert( opRes.type() == CV_64FC1 );
     Mat _dst, dst, diff;
     cv::reduce( src, _dst, dim, opType, dstType );
     _dst.convertTo( dst, CV_64FC1 );
@@ -192,7 +190,7 @@ int Core_ReduceTest::checkCase( int srcType, int dstType, int dim, Size sz )
     else if( srcType == CV_64FC1 )
         testReduce<double>( src, sum, avg, max, min, dim );
     else
-        CV_Assert( 0 );
+        assert( 0 );
 
     // 1. sum
     tempCode = checkOp( src, dstType, CV_REDUCE_SUM, sum, dim );
@@ -954,7 +952,7 @@ void Core_ArrayOpTest::run( int /* start_from */)
 }
 
 
-template <class ElemType>
+template <class T>
 int calcDiffElemCountImpl(const vector<Mat>& mv, const Mat& m)
 {
     int diffElemCount = 0;
@@ -963,12 +961,12 @@ int calcDiffElemCountImpl(const vector<Mat>& mv, const Mat& m)
     {
         for(int x = 0; x < m.cols; x++)
         {
-            const ElemType* mElem = &m.at<ElemType>(y,x*mChannels);
+            const T* mElem = &m.at<T>(y, x*mChannels);
             size_t loc = 0;
             for(size_t i = 0; i < mv.size(); i++)
             {
                 const size_t mvChannel = mv[i].channels();
-                const ElemType* mvElem = &mv[i].at<ElemType>(y,x*(int)mvChannel);
+                const T* mvElem = &mv[i].at<T>(y, x*(int)mvChannel);
                 for(size_t li = 0; li < mvChannel; li++)
                     if(mElem[loc + li] != mvElem[li])
                         diffElemCount++;
@@ -1352,8 +1350,6 @@ TEST(Core_Matx, fromMat_)
     ASSERT_EQ( cvtest::norm(a, b, NORM_INF), 0.);
 }
 
-#ifdef CV_CXX11
-
 TEST(Core_Matx, from_initializer_list)
 {
     Mat_<double> a = (Mat_<double>(2,2) << 10, 11, 12, 13);
@@ -1368,8 +1364,6 @@ TEST(Core_Mat, regression_9507)
     EXPECT_EQ(25u, m2.total());
 }
 
-#endif // CXX11
-
 TEST(Core_InputArray, empty)
 {
     vector<vector<Point> > data;
@@ -1646,7 +1640,6 @@ TEST(Mat, regression_10507_mat_setTo)
     }
 }
 
-#ifdef CV_CXX_STD_ARRAY
 TEST(Core_Mat_array, outputArray_create_getMat)
 {
     cv::Mat_<uchar> src_base(5, 1);
@@ -1735,7 +1728,6 @@ TEST(Core_Mat_array, SplitMerge)
         EXPECT_EQ(0, cvtest::norm(src[i], dst[i], NORM_INF));
     }
 }
-#endif
 
 TEST(Mat, regression_8680)
 {
@@ -1745,8 +1737,6 @@ TEST(Mat, regression_8680)
    ASSERT_EQ(mat.channels(), 2);
 }
 
-#ifdef CV_CXX11
-
 TEST(Mat_, range_based_for)
 {
     Mat_<uchar> img = Mat_<uchar>::zeros(3, 3);
@@ -1808,8 +1798,6 @@ TEST(Mat_, template_based_ptr)
     ASSERT_FLOAT_EQ(66.0f, *(mat.ptr<float>(idx)));
 }
 
-#endif
-
 
 BIGDATA_TEST(Mat, push_back_regression_4158)  // memory usage: ~10.6 Gb
 {
@@ -1971,171 +1959,6 @@ TEST(Core_InputArray, support_CustomType)
     }
 }
 
-
-TEST(Core_InputArray, fetch_MatExpr)
-{
-    Mat a(Size(10, 5), CV_32FC1, 5);
-    Mat b(Size(10, 5), CV_32FC1, 2);
-    MatExpr expr = a * b.t();                    // gemm expression
-    Mat dst;
-    cv::add(expr, Scalar(1), dst);               // invoke gemm() here
-    void* expr_data = expr.a.data;
-    Mat result = expr;                           // should not call gemm() here again
-    EXPECT_EQ(expr_data, result.data);           // expr data is reused
-    EXPECT_EQ(dst.size(), result.size());
-}
-
-
-#ifdef CV_CXX11
-class TestInputArrayRangeChecking {
-    static const char *kind2str(cv::_InputArray ia)
-    {
-        switch (ia.kind())
-        {
-        #define C(x) case cv::_InputArray::x: return #x
-        C(MAT);
-        C(UMAT);
-        C(EXPR);
-        C(MATX);
-        C(STD_VECTOR);
-        C(NONE);
-        C(STD_VECTOR_VECTOR);
-        C(STD_BOOL_VECTOR);
-        C(STD_VECTOR_MAT);
-        C(STD_ARRAY_MAT);
-        C(STD_VECTOR_UMAT);
-        C(CUDA_GPU_MAT);
-        C(STD_VECTOR_CUDA_GPU_MAT);
-        #undef C
-        default:
-            return "<unsupported>";
-        }
-    }
-
-    static void banner(cv::_InputArray ia, const char *label, const char *name)
-    {
-        std::cout << std::endl
-                  << label << " = " << name << ", Kind: " << kind2str(ia)
-                  << std::endl;
-    }
-
-    template<typename I, typename F>
-    static void testA(I ia, F f, const char *mfname)
-    {
-        banner(ia, "f", mfname);
-        EXPECT_THROW(f(ia, -1), cv::Exception)
-            << "f(ia, " << -1 << ") should throw cv::Exception";
-        for (int i = 0; i < int(ia.size()); i++)
-        {
-            EXPECT_NO_THROW(f(ia, i))
-                << "f(ia, " << i << ") should not throw an exception";
-        }
-        EXPECT_THROW(f(ia, int(ia.size())), cv::Exception)
-            << "f(ia, " << ia.size() << ") should throw cv::Exception";
-    }
-
-    template<typename I, typename F>
-    static void testB(I ia, F f, const char *mfname)
-    {
-        banner(ia, "f", mfname);
-        EXPECT_THROW(f(ia, -1), cv::Exception)
-            << "f(ia, " << -1 << ") should throw cv::Exception";
-        for (int i = 0; i < int(ia.size()); i++)
-        {
-            EXPECT_NO_THROW(f(ia, i))
-                << "f(ia, " << i << ") should not throw an exception";
-        }
-        EXPECT_THROW(f(ia, int(ia.size())), cv::Exception)
-            << "f(ia, " << ia.size() << ") should throw cv::Exception";
-    }
-
-    static void test_isContinuous()
-    {
-        auto f = [](cv::_InputArray ia, int i) { (void)ia.isContinuous(i); };
-
-        cv::Mat M;
-        cv::UMat uM;
-
-        std::vector<cv::Mat> vec = {M, M};
-        std::array<cv::Mat, 2> arr = {M, M};
-        std::vector<cv::UMat> uvec = {uM, uM};
-
-        testA(vec, f, "isContinuous");
-        testA(arr, f, "isContinuous");
-        testA(uvec, f, "isContinuous");
-    }
-
-    static void test_isSubmatrix()
-    {
-        auto f = [](cv::_InputArray ia, int i) { (void)ia.isSubmatrix(i); };
-
-        cv::Mat M;
-        cv::UMat uM;
-
-        std::vector<cv::Mat> vec = {M, M};
-        std::array<cv::Mat, 2> arr = {M, M};
-        std::vector<cv::UMat> uvec = {uM, uM};
-
-        testA(vec, f, "isSubmatrix");
-        testA(arr, f, "isSubmatrix");
-        testA(uvec, f, "isSubmatrix");
-    }
-
-    static void test_offset()
-    {
-        auto f = [](cv::_InputArray ia, int i) { return ia.offset(i); };
-
-        cv::Mat M;
-        cv::UMat uM;
-        cv::cuda::GpuMat gM;
-
-        std::vector<cv::Mat> vec = {M, M};
-        std::array<cv::Mat, 2> arr = {M, M};
-        std::vector<cv::UMat> uvec = {uM, uM};
-        std::vector<cv::cuda::GpuMat> gvec = {gM, gM};
-
-        testB(vec, f, "offset");
-        testB(arr, f, "offset");
-        testB(uvec, f, "offset");
-        testB(gvec, f, "offset");
-    }
-
-    static void test_step()
-    {
-        auto f = [](cv::_InputArray ia, int i) { return ia.step(i); };
-
-        cv::Mat M;
-        cv::UMat uM;
-        cv::cuda::GpuMat gM;
-
-        std::vector<cv::Mat> vec = {M, M};
-        std::array<cv::Mat, 2> arr = {M, M};
-        std::vector<cv::UMat> uvec = {uM, uM};
-        std::vector<cv::cuda::GpuMat> gvec = {gM, gM};
-
-        testB(vec, f, "step");
-        testB(arr, f, "step");
-        testB(uvec, f, "step");
-        testB(gvec, f, "step");
-    }
-
-public:
-    static void run()
-    {
-        test_isContinuous();
-        test_isSubmatrix();
-        test_offset();
-        test_step();
-    }
-};
-
-TEST(Core_InputArray, range_checking)
-{
-    TestInputArrayRangeChecking::run();
-}
-#endif
-
-
 TEST(Core_Vectors, issue_13078)
 {
     float floats_[] = { 1, 2, 3, 4, 5, 6, 7, 8 };
@@ -2236,86 +2059,6 @@ TEST(Core_Eigen, eigen2cv_check_Mat_type)
 }
 #endif // HAVE_EIGEN
 
-#ifdef OPENCV_EIGEN_TENSOR_SUPPORT
-TEST(Core_Eigen, cv2eigen_check_tensor_conversion)
-{
-    Mat A(2, 3, CV_32FC3);
-    float value = 0;
-    for(int row=0; row<A.rows; row++)
-        for(int col=0; col<A.cols; col++)
-            for(int ch=0; ch<A.channels(); ch++)
-                A.at<Vec3f>(row,col)[ch] = value++;
-
-    Eigen::Tensor<float, 3, Eigen::RowMajor> row_tensor;
-    cv2eigen(A, row_tensor);
-
-    float* mat_ptr = (float*)A.data;
-    float* tensor_ptr = row_tensor.data();
-    for (int i=0; i< row_tensor.size(); i++)
-        ASSERT_FLOAT_EQ(mat_ptr[i], tensor_ptr[i]);
-
-    Eigen::Tensor<float, 3, Eigen::ColMajor> col_tensor;
-    cv2eigen(A, col_tensor);
-    value = 0;
-    for(int row=0; row<A.rows; row++)
-        for(int col=0; col<A.cols; col++)
-            for(int ch=0; ch<A.channels(); ch++)
-                ASSERT_FLOAT_EQ(value++, col_tensor(row,col,ch));
-}
-#endif // OPENCV_EIGEN_TENSOR_SUPPORT
-
-#ifdef OPENCV_EIGEN_TENSOR_SUPPORT
-TEST(Core_Eigen, eigen2cv_check_tensor_conversion)
-{
-    Eigen::Tensor<float, 3, Eigen::RowMajor> row_tensor(2,3,3);
-    Eigen::Tensor<float, 3, Eigen::ColMajor> col_tensor(2,3,3);
-    float value = 0;
-    for(int row=0; row<row_tensor.dimension(0); row++)
-        for(int col=0; col<row_tensor.dimension(1); col++)
-            for(int ch=0; ch<row_tensor.dimension(2); ch++)
-            {
-                row_tensor(row,col,ch) = value;
-                col_tensor(row,col,ch) = value;
-                value++;
-            }
-
-    Mat A;
-    eigen2cv(row_tensor, A);
-
-    float* tensor_ptr = row_tensor.data();
-    float* mat_ptr = (float*)A.data;
-    for (int i=0; i< row_tensor.size(); i++)
-        ASSERT_FLOAT_EQ(tensor_ptr[i], mat_ptr[i]);
-
-    Mat B;
-    eigen2cv(col_tensor, B);
-
-    value = 0;
-    for(int row=0; row<B.rows; row++)
-        for(int col=0; col<B.cols; col++)
-            for(int ch=0; ch<B.channels(); ch++)
-                ASSERT_FLOAT_EQ(value++, B.at<Vec3f>(row,col)[ch]);
-}
-#endif // OPENCV_EIGEN_TENSOR_SUPPORT
-
-#ifdef OPENCV_EIGEN_TENSOR_SUPPORT
-TEST(Core_Eigen, cv2eigen_tensormap_check_tensormap_access)
-{
-    float arr[] = {0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11};
-    Mat a_mat(2, 2, CV_32FC3, arr);
-    Eigen::TensorMap<Eigen::Tensor<float, 3, Eigen::RowMajor>> a_tensor = cv2eigen_tensormap<float>(a_mat);
-
-    for(int i=0; i<a_mat.rows; i++) {
-        for (int j=0; j<a_mat.cols; j++) {
-            for (int ch=0; ch<a_mat.channels(); ch++) {
-                ASSERT_FLOAT_EQ(a_mat.at<Vec3f>(i,j)[ch], a_tensor(i,j,ch));
-                ASSERT_EQ(&a_mat.at<Vec3f>(i,j)[ch], &a_tensor(i,j,ch));
-            }
-        }
-    }
-}
-#endif // OPENCV_EIGEN_TENSOR_SUPPORT
-
 TEST(Mat, regression_12943)  // memory usage: ~4.5 Gb
 {
     applyTestTag(CV_TEST_TAG_MEMORY_6GB);
@@ -2337,48 +2080,4 @@ TEST(Mat, empty_iterator_16855)
     EXPECT_TRUE(m.begin<uchar>() == m.end<uchar>());
 }
 
-
-TEST(Mat, regression_18473)
-{
-    std::vector<int> sizes(3);
-    sizes[0] = 20;
-    sizes[1] = 50;
-    sizes[2] = 100;
-#if 1  // with the fix
-    std::vector<size_t> steps(2);
-    steps[0] = 50*100*2;
-    steps[1] = 100*2;
-#else  // without the fix
-    std::vector<size_t> steps(3);
-    steps[0] = 50*100*2;
-    steps[1] = 100*2;
-    steps[2] = 2;
-#endif
-    std::vector<short> data(20*50*100, 0);  // 1Mb
-    data[data.size() - 1] = 5;
-
-    // param steps Array of ndims-1 steps
-    Mat m(sizes, CV_16SC1, (void*)data.data(), (const size_t*)steps.data());
-
-    ASSERT_FALSE(m.empty());
-    EXPECT_EQ((int)5, (int)m.at<short>(19, 49, 99));
-}
-
-
-TEST(Mat, ptrVecni_20044)
-{
-    Mat_<int> m(3,4); m << 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12;
-    Vec2i idx(1,1);
-
-    uchar *u = m.ptr(idx);
-    EXPECT_EQ(int(6), *(int*)(u));
-    const uchar *cu = m.ptr(idx);
-    EXPECT_EQ(int(6), *(int*)(cu));
-
-    int *i = m.ptr<int>(idx);
-    EXPECT_EQ(int(6), *(i));
-    const int *ci = m.ptr<int>(idx);
-    EXPECT_EQ(int(6), *(ci));
-}
-
 }} // namespace
